#include <Arduino.h>
#include <SDI12.h>
#include <SoftwareSerial.h>
#include <Wire.h>                

#define I2C_ADDRESSS 99               
#define DATA_PIN 10             
#define POWER_PIN -1            

const byte SENSOR_ADDRESS = 4;

SDI12 mySDI12(DATA_PIN);

String sdiResponse = "";
String Mcommand = String(SENSOR_ADDRESS) + "M";
String Icommand = String(SENSOR_ADDRESS) + "I";
String Dcommand = String(SENSOR_ADDRESS) + "D";

bool checkSDIcommand = false;
float PHfromsensor;

float measurePH(){
  byte code = 0;  
  char ph_data[32];                
  byte in_char = 0;  
  byte i = 0; 
  float ph_float;

  Wire.beginTransmission(I2C_ADDRESSS);                                              
  Wire.write('r');                                                    
  Wire.endTransmission();    
  delay(815);
  Wire.requestFrom(I2C_ADDRESSS, 32, 1); 		                                      
  code = Wire.read();               		                                     

  switch (code) {							          
    case 1:                         		
      Serial.println("Success");    		
      break;                        		

    case 2:                         		
      Serial.println("Failed");     		
      break;                        		

    case 254:                       		
      Serial.println("Pending");    		
      break;                        		

    case 255:                       		
      Serial.println("No Data");    		
      break;                        		
  }
  while (Wire.available() > 0) {        
    in_char = Wire.read();              
    ph_data[i] = in_char;					      
    i += 1;                          		
    if (in_char == 0) {              		
      i = 0;                         		
      break;                         		
    }
  }
  Serial.println("PH:" + String(ph_data));          		//print the data.
  ph_float=atof(ph_data);
  return ph_float;
}

void setup() {
  Wire.begin();
  Serial.begin(115200);
  Serial.println("Opening SDI-12 bus...");
  mySDI12.begin();
  delay(500);  
  pinMode(LED_BUILTIN, OUTPUT);
  mySDI12.forceListen();
}

void loop() {
  while (mySDI12.available() > 0){  
    char c = mySDI12.read();
    if (c != 0) {
      // Serial.print(c);
      if (c == '!') {
        Serial.println("***SDI data: " + String(sdiResponse));
        checkSDIcommand = true;
      }
      else {
        sdiResponse += c;
      }
    }
  }
  if (checkSDIcommand){
    if(sdiResponse == "?") {
      Serial.println("***Sending my addres: " + String(SENSOR_ADDRESS));
      delay(20);
      mySDI12.sendResponse(String(SENSOR_ADDRESS) + "\r\n");
    }
    if(sdiResponse == Icommand) {
      Serial.println("***Sending my information: PH sensor");
      delay(20);
      mySDI12.sendResponse("PH sensor\r\n");
    }
    if(sdiResponse == Mcommand) {
      Serial.println("***Starting measure");
      delay(20);
      mySDI12.sendResponse("00023\r\n");
      PHfromsensor = measurePH();
      mySDI12.sendResponse(String(SENSOR_ADDRESS) + "\r\n");
    }
    if(sdiResponse == Dcommand) {
      Serial.println("***Sending measure");
      delay(20);
      mySDI12.sendResponse(String(SENSOR_ADDRESS) + "+0+" + String(PHfromsensor,3) + "\r\n");
    }
    mySDI12.forceListen();
    sdiResponse = "";
    checkSDIcommand = false;
  }
}